# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: receiver.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import aiddl_pb2 as aiddl__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0ereceiver.proto\x12\x17org.aiddl.external.grpc\x1a\x0b\x61iddl.proto\"\x93\x01\n\x05Query\x12/\n\x07sort_by\x18\x01 \x01(\x0e\x32\x1e.org.aiddl.external.grpc.Order\x12\x32\n\npull_order\x18\x02 \x01(\x0e\x32\x1e.org.aiddl.external.grpc.Order\x12\x10\n\x08pull_max\x18\x03 \x01(\x05\x12\x13\n\x0b\x66lush_queue\x18\x04 \x01(\x08\";\n\x08Messages\x12/\n\x08messages\x18\x01 \x03(\x0b\x32\x1d.org.aiddl.external.grpc.Term*+\n\x05Order\x12\x10\n\x0cOLDEST_FIRST\x10\x00\x12\x10\n\x0cNEWEST_FIRST\x10\x01\x32X\n\x08Receiver\x12L\n\x07Receive\x12\x1e.org.aiddl.external.grpc.Query\x1a!.org.aiddl.external.grpc.MessagesB\x19\n\x17org.aiddl.external.grpcb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'receiver_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\027org.aiddl.external.grpc'
  _ORDER._serialized_start=267
  _ORDER._serialized_end=310
  _QUERY._serialized_start=57
  _QUERY._serialized_end=204
  _MESSAGES._serialized_start=206
  _MESSAGES._serialized_end=265
  _RECEIVER._serialized_start=312
  _RECEIVER._serialized_end=400
# @@protoc_insertion_point(module_scope)



from concurrent import futures

import grpc

import time

from aiddl_core.representation import Sym

from aiddl_external_grpc_python.container.grpc_function import GrpcFunctionLoader
from aiddl_external_grpc_python.converter import Converter

import aiddl_external_grpc_python.generated.container_pb2 as container_pb2
import aiddl_external_grpc_python.generated.container_pb2_grpc as container_pb2_grpc

LOADER_URI = Sym("org.aiddl.network.register-remote-function")


class ContainerServer(container_pb2_grpc.ContainerServicer):
    def __init__(self, port, container, allow_remote_registry=False):
        self.port = port
        self.freg = container.fun_reg
        self.container = container
        self.converter = Converter(container)
        if allow_remote_registry:
            self.freg.add_function(LOADER_URI,
                                   GrpcFunctionLoader(self.freg))

    def FunctionCall(self, request, context):
        uri = Sym(request.function_uri)
        print(f"Calling: {uri}")
        t_start = time.time()
        args = self.converter.pb2aiddl(request.arg)
        print(f"  - converting input: {time.time()-t_start}s")

        f = self.freg.get_function(uri)
        t_start = time.time()
        print(f"  - function call: {time.time()-t_start}s")

        r = f(args)
        result = container_pb2.FunctionCallResult()
        result.status = container_pb2.SUCCESS

        t_start = time.time()
        result_pb = self.converter.aiddl2pb(r)
        print(f"  - converting output: {time.time()-t_start}s")

        t_start = time.time()
        result.result.CopyFrom(result_pb)
        print(f"  - setting output: {time.time()-t_start}s")

        return result

    def start(self):
        self.server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
        container_pb2_grpc.add_ContainerServicer_to_server(
            self, self.server)
        self.server.add_insecure_port('0.0.0.0:%d' % (self.port))
        self.server.start()

    def wait_for_termination(self):
        self.server.wait_for_termination()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: aiddl.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0b\x61iddl.proto\x12\x17org.aiddl.external.grpc\"2\n\x08Rational\x12\x11\n\tnominator\x18\x01 \x01(\x03\x12\x13\n\x0b\x64\x65nominator\x18\x02 \x01(\x03\"w\n\x0e\x43ollectionTerm\x12\x38\n\x07\x63olType\x18\x01 \x01(\x0e\x32\'.org.aiddl.external.grpc.CollectionType\x12+\n\x04\x64\x61ta\x18\x02 \x03(\x0b\x32\x1d.org.aiddl.external.grpc.Term\"b\n\x06KeyVal\x12*\n\x03key\x18\x01 \x01(\x0b\x32\x1d.org.aiddl.external.grpc.Term\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x1d.org.aiddl.external.grpc.Term\"\x92\x01\n\x06\x45ntRef\x12-\n\x06module\x18\x01 \x01(\x0b\x32\x1d.org.aiddl.external.grpc.Term\x12+\n\x04name\x18\x02 \x01(\x0b\x32\x1d.org.aiddl.external.grpc.Term\x12,\n\x05\x61lias\x18\x03 \x01(\x0b\x32\x1d.org.aiddl.external.grpc.Term\"\xa3\x03\n\x04Term\x12\r\n\x03sym\x18\x01 \x01(\tH\x00\x12\r\n\x03var\x18\x02 \x01(\tH\x00\x12\r\n\x03str\x18\x03 \x01(\tH\x00\x12\x11\n\x07\x62oolean\x18\x04 \x01(\x08H\x00\x12\r\n\x03int\x18\x05 \x01(\x03H\x00\x12\x0e\n\x04real\x18\x06 \x01(\x01H\x00\x12\x35\n\x08rational\x18\x07 \x01(\x0b\x32!.org.aiddl.external.grpc.RationalH\x00\x12\x42\n\x0fother_numerical\x18\x08 \x01(\x0e\x32\'.org.aiddl.external.grpc.OtherNumericalH\x00\x12=\n\ncollection\x18\t \x01(\x0b\x32\'.org.aiddl.external.grpc.CollectionTermH\x00\x12.\n\x03kvp\x18\n \x01(\x0b\x32\x1f.org.aiddl.external.grpc.KeyValH\x00\x12\x32\n\x07\x65nt_ref\x18\x0b \x01(\x0b\x32\x1f.org.aiddl.external.grpc.EntRefH\x00\x12\x11\n\x07\x66un_ref\x18\x0c \x01(\tH\x00\x42\x0b\n\tterm_type\"\x1d\n\x08\x41iddlStr\x12\x11\n\taiddl_str\x18\x01 \x01(\t*3\n\x0eOtherNumerical\x12\x0b\n\x07INF_POS\x10\x00\x12\x0b\n\x07INF_NEG\x10\x01\x12\x07\n\x03NAN\x10\x02*.\n\x0e\x43ollectionType\x12\x08\n\x04LIST\x10\x00\x12\x07\n\x03SET\x10\x01\x12\t\n\x05TUPLE\x10\x02\x42\x19\n\x17org.aiddl.external.grpcb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'aiddl_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\027org.aiddl.external.grpc'
  _OTHERNUMERICAL._serialized_start=915
  _OTHERNUMERICAL._serialized_end=966
  _COLLECTIONTYPE._serialized_start=968
  _COLLECTIONTYPE._serialized_end=1014
  _RATIONAL._serialized_start=40
  _RATIONAL._serialized_end=90
  _COLLECTIONTERM._serialized_start=92
  _COLLECTIONTERM._serialized_end=211
  _KEYVAL._serialized_start=213
  _KEYVAL._serialized_end=311
  _ENTREF._serialized_start=314
  _ENTREF._serialized_end=460
  _TERM._serialized_start=463
  _TERM._serialized_end=882
  _AIDDLSTR._serialized_start=884
  _AIDDLSTR._serialized_end=913
# @@protoc_insertion_point(module_scope)



import grpc

from aiddl_core.representation.sym import Sym
from aiddl_core.representation.funref import FunRef
import aiddl_core.parser.parser as parser

import aiddl_external_grpc_python.generated.container_pb2 as container_pb2
import aiddl_external_grpc_python.generated.container_pb2_grpc as container_pb2_grpc
from aiddl_external_grpc_python.converter import Converter


class GrpcFunction:
    def __init__(self, host, port, function_uri, container):
        self.host = host
        self.port = port
        self.converter = Converter(container)
        self.function_uri = str(function_uri)

    def __call__(self, args):
        with grpc.insecure_channel('%s:%d' % (self.host, self.port)) as channel:
            stub = container_pb2_grpc.ContainerStub(channel)
            # print("-------------- Request --------------")
            uri = self.function_uri
            arg = self.converter.aiddl2pb(args)
            req = container_pb2.FunctionCallRequest()
            req.function_uri = uri
            req.arg.CopyFrom(arg)
            answer = stub.FunctionCall(req) # Execute request
            # print("-------------- Answer  --------------")
            # print(answer)
            return self.converter.pb2aiddl(answer.result)


class GrpcFunctionLoader:
    def __init__(self, container):
        self.container = container

    def __call__(self, args):
        uri = args[Sym("uri")]
        host = args[Sym("host")].string_value()
        port = args[Sym("port")].int_value()
        f = GrpcFunction(host, port, uri, self.container)
        self.F.add_function(uri, f)
        return FunRef(uri, self.container.fun_ref)



from concurrent import futures
from abc import ABC, abstractmethod

import grpc

import aiddl_external_grpc_python.generated.sender_pb2_grpc as sender_pb2_grpc


class SenderServer(sender_pb2_grpc.SenderServicer):
    def __init__(self, port):
        self.server = None
        self.port = port

    # @abstractmethod
    def Send(self, request, context):
        pass

    def start(self):
        self.server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
        sender_pb2_grpc.add_SenderServicer_to_server(self, self.server)
        self.server.add_insecure_port(f'0.0.0.0:{self.port}')
        self.server.start()

    def wait_for_termination(self):
        self.server.wait_for_termination()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sender.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import aiddl_pb2 as aiddl__pb2
from . import empty_pb2 as empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0csender.proto\x12\x17org.aiddl.external.grpc\x1a\x0b\x61iddl.proto\x1a\x0b\x65mpty.proto2O\n\x06Sender\x12\x45\n\x04Send\x12\x1d.org.aiddl.external.grpc.Term\x1a\x1e.org.aiddl.external.grpc.EmptyB\x19\n\x17org.aiddl.external.grpcb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'sender_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\027org.aiddl.external.grpc'
  _SENDER._serialized_start=67
  _SENDER._serialized_end=146
# @@protoc_insertion_point(module_scope)



from aiddl_core.container import Container
from aiddl_core.representation import Sym
from aiddl_core.representation import Var
from aiddl_core.representation import Str
from aiddl_core.representation import Int
from aiddl_core.representation import Real
from aiddl_core.representation import Rat
from aiddl_core.representation import Boolean
from aiddl_core.representation import Inf
from aiddl_core.representation import NaN
from aiddl_core.representation import Set
from aiddl_core.representation import List
from aiddl_core.representation import Tuple
from aiddl_core.representation import KeyVal
from aiddl_core.representation import EntRef
from aiddl_core.representation import FunRef

from aiddl_external_grpc_python.generated.aiddl_pb2 import Term as PbTerm
from aiddl_external_grpc_python.generated.aiddl_pb2 import CollectionTerm as PbCollection
from aiddl_external_grpc_python.generated.aiddl_pb2 import NAN
from aiddl_external_grpc_python.generated.aiddl_pb2 import INF_POS
from aiddl_external_grpc_python.generated.aiddl_pb2 import INF_NEG
from aiddl_external_grpc_python.generated.aiddl_pb2 import SET
from aiddl_external_grpc_python.generated.aiddl_pb2 import LIST
from aiddl_external_grpc_python.generated.aiddl_pb2 import TUPLE


class Converter:
    def __init__(self, container: Container):
        self.container = container

    def aiddl2pb(self, x):
        r = PbTerm()
        if isinstance(x, Boolean):
            r.boolean = x.unpack()
        elif isinstance(x, Sym):
            r.sym = x.string
        elif isinstance(x, Var):
            if str(x)[0] == "_":
                r.var = "_"
            else:
                r.var = str(x)[1:]
        elif isinstance(x, Str):
            r.str = x.string

        elif isinstance(x, Int):
            r.int = x.int_value
        elif isinstance(x, Real):
            r.real = x.real_value
        elif isinstance(x, Rat):
            r.rational.nominator = x.nominator
            r.rational.denominator = x.denominator
        elif isinstance(x, Inf):
            if x.is_inf_pos():
                r.other_numerical = INF_POS
            elif x.is_inf_neg():
                r.other_numerical = INF_NEG
        elif isinstance(x, NaN):
            r.other_numerical = NAN
        elif isinstance(x, Set):
            r.collection.colType = SET
            for e in x:
                r.collection.data.append(self.aiddl2pb(e))
        elif isinstance(x, List):
            r.collection.colType = LIST
            for e in x:
                r.collection.data.append(self.aiddl2pb(e))
        elif isinstance(x, Tuple):
            r.collection.colType = TUPLE
            for e in x:
                r.collection.data.append(self.aiddl2pb(e))
        elif isinstance(x, KeyVal):
            r.kvp.key.CopyFrom( self.aiddl2pb(x.key))
            r.kvp.value.CopyFrom(self.aiddl2pb(x.value))
        elif isinstance(x, EntRef):
            r.ent_ref.module.CopyFrom(self.aiddl2pb(x.module))
            r.ent_ref.name.CopyFrom(self.aiddl2pb(x.target))
            r.ent_ref.alias.CopyFrom(self.aiddl2pb(x.alias))
        elif isinstance(x, FunRef):
            r.fun_ref = str(x.function_uri)
        return r

    def pb2aiddl(self, x):
        term_type = x.WhichOneof('term_type')
        if term_type == 'sym':
            return Sym(x.sym)
        elif term_type == 'var':
            if x.var == '_':
                return Var()
            else:
                return Var(x.var)
        elif term_type == 'str':
            return Str(x.str)
        elif term_type == 'boolean':
            return Boolean(x.boolean)
        elif term_type == 'int':
            return Int(x.int)
        elif term_type == 'real':
            return Real(x.real)
        elif term_type == 'rational':
            return Rat(x.rational.nominator, x.rational.denominator)
        elif term_type == 'other_numerical':
            if x.other_numerical == INF_POS:
                return Inf.pos()
            elif x.other_numerical == INF_NEG:
                return Inf.neg()
            elif x.other_numerical == NAN:
                return NaN()
        elif term_type == 'collection':
            if x.collection.colType == SET:
                return Set([self.pb2aiddl(e) for e in x.collection.data])
            elif x.collection.colType == LIST:
                return List([self.pb2aiddl(e) for e in x.collection.data])
            elif x.collection.colType == TUPLE:
                return Tuple([self.pb2aiddl(e) for e in x.collection.data])
        elif term_type == 'kvp':
            return KeyVal(self.pb2aiddl(x.kvp.key),
                          self.pb2aiddl(x.kvp.value))
        elif term_type == 'ent_ref':
            return EntRef(
                self.pb2aiddl(x.ent_ref.name),
                self.pb2aiddl(x.ent_ref.module),
                self.pb2aiddl(x.ent_ref.alias))
        elif term_type == 'fun_ref':
            return FunRef(Sym(x.fun_ref), c.fun_reg)


c = Container()
conv = Converter(c)

x = Sym("x")
assert x == conv.pb2aiddl(conv.aiddl2pb(x))
x = Var("x")
assert x == conv.pb2aiddl(conv.aiddl2pb(x))
x = Var()
assert conv.pb2aiddl(conv.aiddl2pb(x)).is_anonymous
x = Str("This is a string!")
assert x == conv.pb2aiddl(conv.aiddl2pb(x))
x = Boolean(True)
assert x == conv.pb2aiddl(conv.aiddl2pb(x))
x = Int(35)
assert x == conv.pb2aiddl(conv.aiddl2pb(x))
x = Real(35.2345)
assert x == conv.pb2aiddl(conv.aiddl2pb(x))
x = Rat(1, 4)
assert x == conv.pb2aiddl(conv.aiddl2pb(x))
x = Inf.pos()
assert conv.pb2aiddl(conv.aiddl2pb(x)).is_inf_pos
x = Inf.neg()
assert conv.pb2aiddl(conv.aiddl2pb(x)).is_inf_neg
x = NaN()
assert conv.pb2aiddl(conv.aiddl2pb(x)).is_nan
x = Set(Int(1), Sym("a"), Rat(1, 4))
assert x == conv.pb2aiddl(conv.aiddl2pb(x))
x = List(Int(1), Sym("a"), Rat(1, 4))
assert x == conv.pb2aiddl(conv.aiddl2pb(x))
x = Tuple(Int(1), Sym("a"), Rat(1, 4))
assert x == conv.pb2aiddl(conv.aiddl2pb(x))
x = KeyVal(Sym("x"), Int(4))
assert x == conv.pb2aiddl(conv.aiddl2pb(x))
x = EntRef(Sym("m"), Tuple(Sym("x"), Int(1)), Sym("alias"))
assert x == conv.pb2aiddl(conv.aiddl2pb(x))
x = FunRef(Sym("f"), c.fun_reg)
assert x == conv.pb2aiddl(conv.aiddl2pb(x))

